@@ -38,9 +38,9 @@
 
 #include "view_cw.h"
 
-#define	CW_SAMPLERATE	8000
+#define	CW_SAMPLERATE	16000 //8000
+
 #define	CWMaxSymLen		4096		// AG1LE: - was 4096 
-#define CWKNUM 			640			// 1/2 dot length at 5 wpm
 
 #define MAX_MORSE_ELEMENTS 6 // maximum of 6 elements in a Morse character 256
 
@@ -52,19 +52,20 @@
 #define	ASC_SPACE	' '	// ASCII space char 
 
 // Tone and timing magic numbers. 
-#define	KWPM	(12 * CW_SAMPLERATE/10)  // #samples in dot = KWPM / WPM
+#define	KWPM		(12 * CW_SAMPLERATE/10)		// # samples in dot = KWPM / WPM
+#define CWKNUM 		((KWPM) / 10)				// 640, 1/2 dot length at 5 wpm
 
 #define	TONE_SILENT	0	// 0Hz = silent 'tone' 
 #define	USECS_PER_SEC	1000000	// Microseconds in a second 
 
-#define	INITIAL_SEND_SPEED	18	// Initial send speed in WPM 
+#define	INITIAL_SEND_SPEED		18	// Initial send speed in WPM 
 #define	INITIAL_RECEIVE_SPEED	18	// Initial receive speed in WPM 
 
 // Initial adaptive speed threshold 
-#define	INITIAL_THRESHOLD	((KWPM / INITIAL_RECEIVE_SPEED) * 2)
+#define	INITIAL_THRESHOLD	(((KWPM) / INITIAL_RECEIVE_SPEED) * 2)
 
 // Initial noise filter threshold 
-#define	INITIAL_NOISE_THRESHOLD	((KWPM / CW_MAX_SPEED) / 2)
+#define	INITIAL_NOISE_THRESHOLD	(((KWPM) / CW_MAX_SPEED) / 2)
 
 #define TRACKING_FILTER_SIZE 16
 
@@ -147,10 +148,12 @@ protected:
 	int lastsym;					// last symbol sent
 	double risetime;			    	// leading/trailing edge rise time (msec)
 	int knum;					// number of samples on edges
+	int qnum;					// number of samples on QSK signal edges
 	int QSKshape;                   		// leading/trailing edge shape factor
 	double qskbuf[OUTBUFSIZE];			// signal array for qsk drive
 	double qskphase;				//
 	bool firstelement;
+	bool lastelement;
 	double maxval;
 
 //	double *keyshape;				// array defining leading edge
@@ -181,7 +184,7 @@ protected:
 	void	reset_rx_filter();
 	int		handle_event(int cw_event, std::string &sc);
 	inline	int usec_diff(unsigned int earlier, unsigned int later);
-	void	send_symbol(int symbol, int len);
+	void	send_symbol(int symbol, int len, int state);
 	void	send_ch(int c);
 	bool 	tables_init();
 	unsigned int tokenize_representation(char *representation);
@@ -208,21 +211,18 @@ protected:
 // transmit wave shaping
 	int		nusymbollen;
 	int		nufsymlen;
-	int		wpm;
-	int		fwpm;
+	int		kpre;
+	int		kpost;
 
-	void	makeshape();
+	double	wpm;
+	double	fwpm;
+
+	void	create_edges();
 	void	sync_transmit_parameters();
+	void	flush_audio();
 
-	fftfilt	*cw_xmt_filter;
-	double	nbfreq;
-	double	nbpf;
-	double lwr;
-	double upr;
-	double	*xmt_signal;
-	double	*qsk_signal;
-	int		qsk_ptr;
-	void nb_filter(double *output, double *qsk, int len);
+	void send_CW_KEYLINE(int);
+	void CW_KEYLINE(bool);
 
 	view_cw	viewcw;
 
@@ -251,4 +251,9 @@ public:
 
 extern bool CW_table_changed;
 
+extern bool CW_KEYLINE_isopen;
+extern void close_CW_KEYLINE();
+extern int open_CW_KEYLINE();
+extern bool CW_KEYLINE_isopen;
+
 #endif
