@@ -90,6 +90,12 @@ protected:
 
 	unsigned cap;
 
+// Audio output
+	std::string audio_filename;
+	bool play_audio;
+
+// CWID
+	bool CW_EOT;
 public:
 	modem();
 	virtual ~modem(){};
@@ -100,15 +106,10 @@ public:
 	virtual void rx_init () = 0;
 	virtual void restart () = 0;
 	virtual void rx_flush() {};
-	virtual int  tx_process ()// = 0;
-	{
-		if (!macro_video_text.empty()) {
-			wfid_text(macro_video_text);
-			macro_video_text.clear();
-		}
-		return 0;
-	}
+	virtual int  tx_process ();
 	virtual int  rx_process (const double *, int len) = 0;
+	virtual void Audio_filename(std::string nm) { audio_filename = nm; play_audio = true; }
+
 	virtual void shutdown(){};
 	virtual void set1(int, int){};
 	virtual void set2(int, int){};
@@ -182,6 +183,7 @@ public:
 	double		get_cwXmtWPM();
 	void		set_cwXmtWPM(double);
 	double		get_cwRcvWPM();
+	virtual void		CW_KEYLINE(bool) {};
 
 	virtual	void		incWPM() {};
 	virtual void		decWPM() {};
@@ -222,6 +224,8 @@ public:
 	void	cwid_send_ch(int ch);
 	void	cwid_sendtext (const std::string& s);
 	void	cwid();
+	void	set_CW_EOT() { CW_EOT = true; }
+	void	clear_CW_EOT() { CW_EOT = false; }
 
 // for fft scan modem
 public:
