@@ -99,6 +99,7 @@ struct status {
 	float	repeatIdleTime;
 	int		timer;
 	int		timerMacro;
+	bool	skip_sked_macro;
 	std::string	LastMacroFile;
 	int		n_rsids;
 
@@ -255,6 +256,8 @@ struct status {
 	bool	nanoCW_online;
 	bool	nanoFSK_online;
 
+	bool	useCW_KEYLINE;
+
 //----------------------------------------------------------------------
 
 	bool	bLastStateRead;
