@@ -751,15 +751,27 @@ void delayed_startup(void *)
 		if (open_nanoIO()) btn_nanoIO_connect->value(1);
 	}
 
+	if (progStatus.useCW_KEYLINE) {
+		if (!open_CW_KEYLINE())
+			progStatus.useCW_KEYLINE = false;
+	}
+
 	if (progdefaults.check_for_updates)
 		cb_mnuCheckUpdate((Fl_Widget *)0, NULL);
 
 #if USE_PORTAUDIO
-	LOG_INFO("%s", str_pa_devices.c_str());
 	try {
+		audio_alert = 0;
 		audio_alert = new Caudio_alert;
 	} catch (...) {
+		audio_alert = 0;
+		LOG_ERROR("%s", "Failed to create audio alert object");
 	}
+
+	if (audio_alert)
+		LOG_INFO("%s", "Created audio alert object");
+
+	reset_audio_alerts();
 #endif
 
 }
@@ -1158,6 +1170,19 @@ int main(int argc, char ** argv)
 	progdefaults.initInterface();
 	trx_start();
 
+#if USE_PORTAUDIO
+	try {
+		audio_alert = 0;
+		audio_alert = new Caudio_alert;
+	} catch (...) {
+		audio_alert = 0;
+		LOG_ERROR("%s", "Failed to create audio alert object");
+	}
+
+	if (audio_alert)
+		LOG_INFO("%s", "Created audio alert object");
+#endif
+
 	if (!have_config) {
 		show_wizard(argc, argv);
 		Fl_Window* w;
@@ -1166,14 +1191,18 @@ int main(int argc, char ** argv)
 	}
 
 	dlgViewer = createViewer();
+
 	dxcluster_viewer = dxc_window();
+	dxcluster_viewer->hide();
+
 	if (!dlgLogbook)
 		create_logbook_dialogs();
 
-	dxcluster_viewer->hide();
-
 	LOGBOOK_colors_font();
 
+	rxaudio_dialog = make_rxaudio_dialog();
+	rxaudio_dialog->hide();
+
 	if( progdefaults.kml_save_dir.empty() ) {
 		progdefaults.kml_save_dir = KmlDir ;
 	}
@@ -1184,11 +1213,16 @@ int main(int argc, char ** argv)
 // the main window will not be restored at its exact saved position if
 // we move it *after* it has been shown.
 #ifndef __APPLE__
-	fl_digi_main->show(argc, argv);
 	progStatus.initLastState();
+	fl_digi_main->show(argc, argv);
 #else
+#  if FLDIGI_FLTK_API_MAJOR == 1 && FLDIGI_FLTK_API_MINOR < 4
+	fl_digi_main->show(argc, argv);
+	progStatus.initLastState();
+#  else
 	progStatus.initLastState();
 	fl_digi_main->show(argc, argv);
+#  endif
 #endif
 
 	if (iconified)
