@@ -143,6 +143,7 @@ status progStatus = {
 	0,					// float	repeatIdleTime;
 	0,					// int timer
 	0,					// int timerMacro
+	false,				// bool skip_sked_macro
 	"macros.mdf",		// string LastMacroFile;
 	0,					// int n_rsids
 	false,				// bool spot_recv
@@ -297,6 +298,7 @@ status progStatus = {
 	false,			// 	bool	nanoCW_online;
 	false,			// 	bool	nanoFSK_online;
 
+	false,			// 	bool	useCW_KEYLINE;
 //----------------------------------------------------------------------
 	false				// bool bLastStateRead;
 };
@@ -673,6 +675,7 @@ if (!bWF_only) {
 	spref.set("nanoCW_online", nanoCW_online);
 	spref.set("nanoFSK_online", nanoFSK_online);
 
+	spref.set("useCW_KEYLINE", useCW_KEYLINE);
 
 }
 
@@ -987,6 +990,7 @@ void status::loadLastState()
 		spref.get("nanoCW_online", i, nanoCW_online); nanoCW_online = i;
 		spref.get("nanoFSK_online", i, nanoFSK_online); nanoFSK_online = i;
 
+		spref.get("useCW_KEYLINE", i, useCW_KEYLINE); useCW_KEYLINE = i;
 }
 
 void status::initLastState()
