@@ -43,6 +43,7 @@
 
 #include "status.h"
 #include "debug.h"
+#include "audio_alert.h"
 
 using namespace std;
 
@@ -273,6 +274,8 @@ modem::modem()
 	s2n_valid = false;
 
 	bandwidth = 0.0;
+
+	CW_EOT = false;
 }
 
 // modem types CW and RTTY do not use the base init()
@@ -292,8 +295,13 @@ void modem::set_freq(double freq)
 		freq,
 		progdefaults.LowFreqCutoff + bandwidth / 2,
 		progdefaults.HighFreqCutoff - bandwidth / 2);
+
 	if (freqlock == false)
 		tx_frequency = frequency;
+
+	if (progdefaults.RxFilt_track_wf)
+		center_rxfilt_at_track();
+
 	REQ(put_freq, frequency);
 }
 
@@ -396,9 +404,19 @@ void modem::set_samplerate(int smprate)
 
 double modem::PTTnco()
 {
-	double amp = sin(PTTphaseacc);
-	PTTphaseacc += TWOPI * 1000 / samplerate;
+	double amp;
+// sine wave PTT signal
+	amp = 0.9 * sin(PTTphaseacc);
+
+// square wave PTT signal
+//	if (PTTphaseacc > M_PI)
+//		amp = - 0.5;
+//	else
+//		amp = 0.5;
+
+	PTTphaseacc += TWOPI * progdefaults.QSKfrequency / samplerate;
 	if (PTTphaseacc > TWOPI) PTTphaseacc -= TWOPI;
+
 	return amp;
 }
 
@@ -581,6 +599,24 @@ void modem::ModulateStereo(double *left, double *right, int len, bool sample_fla
 
 }
 
+//------------------------------------------------------------------------------
+// tx process
+//------------------------------------------------------------------------------
+int  modem::tx_process ()
+{
+	if (!macro_video_text.empty()) {
+		wfid_text(macro_video_text);
+		macro_video_text.clear();
+	}
+	if (play_audio) {
+		disable_modem = true;
+		TXscard->Audio(audio_filename);
+		play_audio = false;
+		disable_modem = false;
+	}
+	return 0;
+}
+
 //------------------------------------------------------------------------------
 // modulate video signal
 //------------------------------------------------------------------------------
@@ -846,12 +882,13 @@ void modem::cwid_sendtext (const string& s)
 
 void modem::cwid()
 {
-	if (progdefaults.cwid_modes.test(mode) &&
-		(progdefaults.CWid == true || progdefaults.macroCWid == true)) {
+	if ((CW_EOT && progdefaults.cwid_modes.test(mode) &&
+		progdefaults.CWid == true) || progdefaults.macroCWid == true) {
 		string tosend = " DE ";
 		tosend += progdefaults.myCall;
 		cwid_sendtext(tosend);
 		progdefaults.macroCWid = false;
+		CW_EOT = false;
 	}
 }
 
