@@ -49,10 +49,9 @@
 #include "macros.h"
 #include "rigsupport.h"
 #include "psm/psm.h"
-
 #include "icons.h"
-
 #include "fft-monitor.h"
+#include "audio_alert.h"
 
 extern fftmon *fft_modem;
 
@@ -330,6 +329,10 @@ void trx_trx_receive_loop()
 					fft_modem->rx_process(rbvec[0].buf, numread);
 
 				active_modem->rx_process(rbvec[0].buf, numread);
+
+				if (audio_alert)
+					audio_alert->monitor(rbvec[0].buf, numread, current_RXsamplerate);
+
 				if (progdefaults.rsid)
 					ReedSolomon->receive(fbuf, numread);
 				dtmf->receive(fbuf, numread);
@@ -424,9 +427,11 @@ void trx_trx_transmit_loop()
 				try {
 					if (!progdefaults.DTMFstr.empty())
 						dtmf->send();
-					if (active_modem->tx_process() < 0)
+					if (active_modem->tx_process() < 0) {
+						active_modem->cwid();
 						if (trx_state != STATE_ABORT)
 							trx_state = STATE_RX;
+					}
 				}
 				catch (const SndException& e) {
 					if (TXscard) TXscard->Close();
@@ -503,17 +508,21 @@ void trx_tune_loop()
 		active_modem->tx_init();
 
 		try {
-			if ((active_modem->get_mode() == MODE_CW)  && (use_nanoIO || progStatus.WK_online)) {
-				if (use_nanoIO) nanoCW_tune(1);
-				else WK_tune(1);
+			if (active_modem->get_mode() == MODE_CW && 
+				(use_nanoIO ||
+				 progStatus.WK_online || 
+				 CW_KEYLINE_isopen)) {
+					if (CW_KEYLINE_isopen) active_modem->CW_KEYLINE(1);
+					else if (use_nanoIO) nanoCW_tune(1);
+					else WK_tune(1);
 
-				REQ(&waterfall::set_XmtRcvBtn, wf, true);
-				while (trx_state == STATE_TUNE) MilliSleep(10);
+					REQ(&waterfall::set_XmtRcvBtn, wf, true);
+					while (trx_state == STATE_TUNE) MilliSleep(10);
 
-				if (use_nanoIO) nanoCW_tune(0);
-				else WK_tune(0);
-			}
-			else {
+					if (CW_KEYLINE_isopen) active_modem->CW_KEYLINE(0);
+					else if (use_nanoIO) nanoCW_tune(0);
+					else WK_tune(0);
+			} else {
 				while (trx_state == STATE_TUNE) {
 					if (_trx_tune == 0) {
 						REQ(&waterfall::set_XmtRcvBtn, wf, true);
